@@ -454,7 +454,7 @@ out:
  * Read in the config file for a device.
  *
  */
-void *nm_system_device_get_system_config (NMDevice *dev)
+void *nm_system_device_get_system_config (NMDevice *dev, NMData *app_data)
 {
 	char *				cfg_file_path = NULL;
 	shvarFile *			file;
@@ -930,3 +930,15 @@ gboolean nm_system_should_modify_resolv_conf (void)
 {
 	return TRUE;
 }
+
+
+/*
+ * nm_system_get_mtu
+ *
+ * Return a user-provided or system-mandated MTU for this device or zero if
+ * no such MTU is provided.
+ */
+unsigned int nm_system_get_mtu (NMDevice *dev)
+{
+	return 0;
+}
