@@ -354,6 +354,7 @@ typedef struct SuSEDeviceConfigData
 	NMIP4Config *	config;
 	gboolean		use_dhcp;
 	gboolean		system_disabled;
+	unsigned int	mtu;
 } SuSEDeviceConfigData;
 
 /*
@@ -521,7 +522,18 @@ found:
 		free (buf);
 	}
 
-	if ((buf = svGetValue (file, "WIRELESS_ESSID")))
+	if ((buf = svGetValue (file, "MTU")))
+	{
+		unsigned long mtu;
+
+		errno = 0;
+		mtu = strtoul (buf, NULL, 10);
+		if (!errno && mtu > 500 && mtu < INT_MAX)
+			sys_data->mtu = (unsigned int) mtu;
+		free (buf);
+	}
+
+	if ((buf = svGetValue (file, "WIRELESS_ESSID")) && strlen (buf) > 1)
 	{
 		NMAccessPoint *	ap;
 		NMAccessPoint *	list_ap;
@@ -545,8 +557,11 @@ found:
 				nm_ap_set_capabilities (ap, NM_802_11_CAP_PROTO_WPA);
 				security = nm_ap_security_wpa_psk_new_from_ap (ap, NM_AUTH_TYPE_WPA_PSK_AUTO);
 				hash = ieee_802_11_cipher_hash (cipher, buf, key);
-				nm_ap_security_set_key (NM_AP_SECURITY (security), hash, strlen (hash));
-				nm_ap_set_security (ap, NM_AP_SECURITY (security));
+				if (hash)
+				{
+					nm_ap_security_set_key (NM_AP_SECURITY (security), hash, strlen (hash));
+					nm_ap_set_security (ap, NM_AP_SECURITY (security));
+				}
 
 				ieee_802_11_cipher_unref (cipher);
 				g_object_unref (G_OBJECT (security));
@@ -594,14 +609,23 @@ found:
 				}
 				else
 				{
+					char **keyv;
+
 					cipher = cipher_wep128_hex_new ();
-					real_key = key;
+
+					keyv = g_strsplit (key, "-", 0);
+					real_key = g_strjoinv (NULL, keyv);
+					g_strfreev (keyv);
+					free (key);
 				}
 				security = nm_ap_security_wep_new_from_ap (ap, IW_AUTH_CIPHER_WEP104);
 			}
 			hash = ieee_802_11_cipher_hash (cipher, buf, real_key);
-			nm_ap_security_set_key (NM_AP_SECURITY (security), hash, strlen (hash));
-			nm_ap_set_security (ap, NM_AP_SECURITY (security));
+			if (hash)
+			{
+				nm_ap_security_set_key (NM_AP_SECURITY (security), hash, strlen (hash));
+				nm_ap_set_security (ap, NM_AP_SECURITY (security));
+			}
 
 			ieee_802_11_cipher_unref (cipher);
 			g_object_unref (G_OBJECT (security));
@@ -751,6 +775,9 @@ out:
 	nm_debug ("mask=%s", ip_str);
 	g_free (ip_str);
 
+	if (sys_data->mtu)
+		nm_debug ("mtu=%u", sys_data->mtu);
+
 	len = nm_ip4_config_get_num_nameservers (sys_data->config);
 	for (i = 0; i < len; i++)
 	{
@@ -1245,3 +1272,20 @@ out_gfree:
 	return ret;
 }
 
+
+/*
+ * nm_system_get_mtu
+ *
+ * Return a user-provided or system-mandated MTU for this device or zero if
+ * no such MTU is provided.
+ */
+unsigned int nm_system_get_mtu (NMDevice *dev)
+{
+	SuSEDeviceConfigData *	sys_data;
+
+	sys_data = nm_device_get_system_config_data (dev);
+	if (!sys_data)
+		return 0;
+
+	return sys_data->mtu;
+}
