@@ -212,9 +212,11 @@ nm_device_new (const char *iface,
 	if (NM_DEVICE_GET_CLASS (dev)->init)
 		NM_DEVICE_GET_CLASS (dev)->init (dev);
 
+	/* This ref should logically go in nm_device_worker, but we need the
+	   ref to be taken before the worker thread is scheduled on a cpu. */
+	g_object_ref (G_OBJECT (dev));
 	dev->priv->worker = g_thread_create (nm_device_worker, dev, TRUE, NULL);
 	g_assert (dev->priv->worker);
-	g_object_ref (G_OBJECT (dev));	/* For the worker thread */
 
 	/* Block until our device thread has actually had a chance to start. */
 	args[0] = &dev->priv->worker_started;
@@ -1189,6 +1191,7 @@ nm_device_activate_stage5_ip_config_commit (NMActRequest *req)
 		nm_system_restart_mdns_responder ();
 		nm_system_set_hostname (self->priv->ip4_config);
 		nm_system_activate_nis (self->priv->ip4_config);
+		nm_system_set_mtu (self);
 		if (NM_DEVICE_GET_CLASS (self)->update_link)
 			NM_DEVICE_GET_CLASS (self)->update_link (self);
 		nm_policy_schedule_activation_finish (req);
@@ -1876,8 +1879,6 @@ nm_device_dispose (GObject *object)
 	 * reference.
 	 */
 
-	nm_device_stop (self);
-
 	nm_system_device_free_system_config (self, self->priv->system_config_data);
 	if (self->priv->ip4_config)
 	{
